@@ -1,5 +1,5 @@
 /*
- * The ARP Scanner (arp-scan) is Copyright (C) 2005-2016 Roy Hills,
+ * The ARP Scanner (arp-scan) is Copyright (C) 2005-2019 Roy Hills,
  * NTA Monitor Ltd.
  *
  * This file is part of arp-scan.
@@ -95,8 +95,8 @@ main(int argc, char *argv[]) {
    struct timeval now;
    struct timeval diff;         /* Difference between two timevals */
    int select_timeout;          /* Select timeout */
-   ARP_UINT64 loop_timediff;    /* Time since last packet sent in us */
-   ARP_UINT64 host_timediff; /* Time since last pkt sent to this host (us) */
+   uint64_t loop_timediff;    /* Time since last packet sent in us */
+   uint64_t host_timediff; /* Time since last pkt sent to this host (us) */
    struct timeval last_packet_time;     /* Time last packet was sent */
    int req_interval;		/* Requested per-packet interval */
    int cum_err=0;               /* Cumulative timing error */
@@ -114,10 +114,11 @@ main(int argc, char *argv[]) {
    bpf_u_int32 netmask;
    bpf_u_int32 localnet;
    int datalink;
-   int get_addr_status = 0;
+   int ret_status = 0;
    int pcap_fd;			/* Pcap file descriptor */
    unsigned char interface_mac[ETH_ALEN];
    pcap_t *pcap_handle;		/* pcap handle */
+   struct in_addr interface_ip_addr;
 /*
  *      Initialise file names to the empty string.
  */
@@ -134,29 +135,71 @@ main(int argc, char *argv[]) {
  */
    Gettimeofday(&start_time);
 /*
- *	Obtain network interface details unless we're reading
- *	from a pcap file or writing to a binary file.
+ * Open the network device for reading with pcap, or the pcap file if we
+ * have specified --readpktfromfile. If we are writing packets to a binary
+ * file, then set pcap_handle to NULL as we don't need to read packets in
+ * this case.
  */
-   if (!pkt_read_file_flag && !pkt_write_file_flag) {
-/*
- *	Determine network interface to use. If the interface was specified
- *	with the --interface option then use that, otherwise use
- *	pcap_lookupdev() to pick a suitable interface.
- */
-   if (!if_name) {
-      if (!(if_name=pcap_lookupdev(errbuf))) {
-         err_msg("pcap_lookupdev: %s", errbuf);
+   if (pkt_read_file_flag) {
+      if (!(pcap_handle = pcap_open_offline(pkt_filename, errbuf)))
+         err_msg("pcap_open_offline: %s", errbuf);
+   } else if (!pkt_write_file_flag) {
+      /*
+       * Determine network interface to use. If the interface was specified
+       * with the --interface option then use that, otherwise use
+       * my_lookupdev() to pick a suitable interface.
+       */
+      if (!if_name) {
+         if (!(if_name=my_lookupdev(errbuf))) {
+            err_msg("my_lookupdev: %s", errbuf);
+         }
       }
-   }
-/*
- *	Obtain the MAC address for the selected interface, and use this
- *	as default for the source hardware addresses in the frame header
- *	and ARP packet if the user has not specified their values.
- *
- *	Die with an error if we can't get the MAC address, as this
- *	indicates that the interface doesn't have a MAC address, so is
- *	probably not a compatible interface type.
- */
+      if (!(pcap_handle = pcap_create(if_name, errbuf)))
+         err_msg("pcap_create: %s", errbuf);
+      if ((pcap_set_snaplen(pcap_handle, snaplen)) < 0)
+         err_msg("pcap_set_snaplen: %s", pcap_geterr(pcap_handle));
+      if ((pcap_set_promisc(pcap_handle, PROMISC)) < 0)
+         err_msg("pcap_set_promisc: %s", pcap_geterr(pcap_handle));
+      if ((pcap_set_immediate_mode(pcap_handle, 1)) < 0)
+         err_msg("pcap_set_immediate_mode: %s", pcap_geterr(pcap_handle));
+      if ((pcap_set_timeout(pcap_handle, TO_MS)) < 0) /* Is this still needed? */
+         err_msg("pcap_set_timeout: %s", pcap_geterr(pcap_handle));
+      ret_status = pcap_activate(pcap_handle);
+      if (ret_status < 0) {		/* Error from pcap_activate() */
+         char *cp;
+
+         cp = pcap_geterr(pcap_handle);
+         if (ret_status == PCAP_ERROR)
+            err_msg("pcap_activate: %s", cp);
+         else if ((ret_status == PCAP_ERROR_NO_SUCH_DEVICE ||
+                   ret_status == PCAP_ERROR_PERM_DENIED) && *cp != '\0')
+            err_msg("pcap_activate: %s: %s\n(%s)", if_name,
+                    pcap_statustostr(ret_status), cp);
+         else
+            err_msg("pcap_activate: %s: %s", if_name,
+                    pcap_statustostr(ret_status));
+      } else if (ret_status > 0) {	/* Warning from pcap_activate() */
+         char *cp;
+
+         cp = pcap_geterr(pcap_handle);
+         if (ret_status == PCAP_WARNING)
+            warn_msg("pcap_activate: %s", cp);
+         else if (ret_status == PCAP_WARNING_PROMISC_NOTSUP && *cp != '\0')
+            warn_msg("pcap_activate: %s: %s\n(%s)", if_name,
+                     pcap_statustostr(ret_status), cp);
+         else
+            warn_msg("pcap_activate: %s: %s", if_name,
+                     pcap_statustostr(ret_status));
+      }
+      /*
+       * Obtain the MAC address for the selected interface, and use this
+       * as the default value for the source hardware addresses in the frame
+       * header and ARP packet if the user has not specified their values.
+       *
+       * Die with an error if we can't get the MAC address, as this
+       * indicates that the interface doesn't have a MAC address, so is
+       * probably not a compatible interface type.
+       */
       get_hardware_address(if_name, interface_mac);
       if (interface_mac[0]==0 && interface_mac[1]==0 &&
           interface_mac[2]==0 && interface_mac[3]==0 &&
@@ -168,43 +211,24 @@ main(int argc, char *argv[]) {
          memcpy(source_mac, interface_mac, ETH_ALEN);
       if (arp_sha_flag == 0)
          memcpy(arp_sha, interface_mac, ETH_ALEN);
-/*
- *	If the user has not specified the ARP source address, obtain the
- *	interface IP address and use that as the default value.
- */
+      /*
+       * Obtain the interface IP address, and use that as the default value
+       * if the user has not manually specified the ARP source address.
+       *
+       * Give a warning and use 0.0.0.0 if the interface has no IP address.
+       */
+      ret_status = get_source_ip(if_name, &interface_ip_addr);
       if (arp_spa_flag == 0) {
-         get_addr_status = get_source_ip(if_name, &arp_spa);
-         if (get_addr_status == -1) {
+         if (ret_status == -1) {
             warn_msg("WARNING: Could not obtain IP address for interface %s. "
                      "Using 0.0.0.0 for", if_name);
-            warn_msg("the source address, which is probably not what you want.");
+            warn_msg("the source address, which may not be what you want.");
             warn_msg("Either configure %s with an IP address, or manually specify"
                      " the address", if_name);
             warn_msg("with the --arpspa option.");
-            memset(&arp_spa, '\0', sizeof(arp_spa));
          }
+         memcpy(&arp_spa, &(interface_ip_addr.s_addr), sizeof(arp_spa));
       }
-   }
-/*
- *	Open the network device for reading with pcap, or the pcap file if we
- *	have specified --readpktfromfile. If we are writing packets to a binary
- *	file, then set pcap_handle to NULL as we don't need to read packets in
- *	this case.
- */
-   if (pkt_read_file_flag) {
-      if (!(pcap_handle = pcap_open_offline(pkt_filename, errbuf)))
-         err_msg("pcap_open_offline: %s", errbuf);
-   } else if (!pkt_write_file_flag) {
-      if (!(pcap_handle = pcap_create(if_name, errbuf)))
-         err_msg("pcap_create: %s", errbuf);
-      if ((pcap_set_snaplen(pcap_handle, snaplen)) < 0)
-         err_msg("pcap_set_snaplen: %s", pcap_geterr(pcap_handle));
-      if ((pcap_set_promisc(pcap_handle, PROMISC)) < 0)
-         err_msg("pcap_set_promisc: %s", pcap_geterr(pcap_handle));
-      if ((pcap_set_timeout(pcap_handle, TO_MS)) < 0)
-         err_msg("pcap_set_timeout: %s", pcap_geterr(pcap_handle));
-      if ((pcap_activate(pcap_handle)) < 0)
-         err_msg("pcap_activate: %s", pcap_geterr(pcap_handle));
    } else {
       pcap_handle = NULL;
    }
@@ -215,10 +239,16 @@ main(int argc, char *argv[]) {
       if ((datalink=pcap_datalink(pcap_handle)) < 0)
          err_msg("pcap_datalink: %s", pcap_geterr(pcap_handle));
       if (!plain_flag) {
-         printf("Interface: %s, datalink type: %s (%s)\n",
-                pkt_read_file_flag ? "savefile" : if_name,
-                pcap_datalink_val_to_name(datalink),
-                pcap_datalink_val_to_description(datalink));
+         if (!pkt_read_file_flag) {
+            printf("Interface: %s, type: %s, "
+                   "MAC: %.2x:%.2x:%.2x:%.2x:%.2x:%.2x, IPv4: %s\n",
+                   if_name, pcap_datalink_val_to_name(datalink),
+                   interface_mac[0], interface_mac[1], interface_mac[2],
+                   interface_mac[3], interface_mac[4], interface_mac[5],
+                   (interface_ip_addr.s_addr==0) ? "(none)" : my_ntoa(interface_ip_addr));
+         } else {
+            printf("Interface: pcap file\n");
+         }
       }
       if (datalink != DLT_EN10MB) {
          warn_msg("WARNING: Unsupported datalink type");
@@ -237,33 +267,7 @@ main(int argc, char *argv[]) {
          err_msg("pcap_fileno: %s", pcap_geterr(pcap_handle));
       if ((pcap_setnonblock(pcap_handle, 1, errbuf)) < 0)
          err_msg("pcap_setnonblock: %s", errbuf);
-/*
- * For the BPF pcap implementation, set the BPF device into immediate mode,
- * otherwise it will buffer the responses.
- */
-#ifdef ARP_PCAP_BPF
-#ifdef BIOCIMMEDIATE
-      {
-         unsigned int one = 1;
 
-         if (ioctl(pcap_fd, BIOCIMMEDIATE, &one) < 0)
-            err_sys("ioctl BIOCIMMEDIATE");
-      }
-#endif /* BIOCIMMEDIATE */
-#endif /* ARP_PCAP_BPF */
-/*
- * For the DLPI pcap implementation on Solaris, set the bufmod timeout to
- * zero. This has the side-effect of setting the chunk size to zero as
- * well, so bufmod will pass all incoming messages on immediately.
- */
-#ifdef ARP_PCAP_DLPI
-      {
-         struct timeval time_zero = {0, 0};
-
-         if (ioctl(pcap_fd, SBIOCSTIME, &time_zero) < 0)
-            err_sys("ioctl SBIOCSTIME");
-      }
-#endif
       if (pcap_lookupnet(if_name, &localnet, &netmask, errbuf) < 0) {
          memset(&localnet, '\0', sizeof(localnet));
          memset(&netmask, '\0', sizeof(netmask));
@@ -483,7 +487,7 @@ main(int argc, char *argv[]) {
       if (packet_out_len < MINIMUM_FRAME_SIZE)
          packet_out_len = MINIMUM_FRAME_SIZE;   /* Adjust to minimum size */
       packet_out_len += PACKET_OVERHEAD;	/* Add layer 2 overhead */
-      interval = ((ARP_UINT64)packet_out_len * 8 * 1000000) / bandwidth;
+      interval = ((uint64_t)packet_out_len * 8 * 1000000) / bandwidth;
       if (verbose > 1) {
          warn_msg("DEBUG: pkt len=%u bytes, bandwidth=%u bps, interval=%u us",
                   packet_out_len, bandwidth, interval);
@@ -520,7 +524,7 @@ main(int argc, char *argv[]) {
  *      potentially send a packet to the current host.
  */
       timeval_diff(&now, &last_packet_time, &diff);
-      loop_timediff = (ARP_UINT64)1000000*diff.tv_sec + diff.tv_usec;
+      loop_timediff = (uint64_t)1000000*diff.tv_sec + diff.tv_usec;
       if (loop_timediff >= (unsigned)req_interval) {
 /*
  *      If the last packet to this host was sent more than the current
@@ -528,7 +532,7 @@ main(int argc, char *argv[]) {
  *      to it.
  */
          timeval_diff(&now, &((*cursor)->last_send_time), &diff);
-         host_timediff = (ARP_UINT64)1000000*diff.tv_sec + diff.tv_usec;
+         host_timediff = (uint64_t)1000000*diff.tv_sec + diff.tv_usec;
          if (host_timediff >= (*cursor)->timeout) {
             if (reset_cum_err) {
                cum_err = 0;
@@ -560,7 +564,7 @@ main(int argc, char *argv[]) {
                remove_host(cursor);     /* Automatically calls advance_cursor() */
                if (first_timeout) {
                   timeval_diff(&now, &((*cursor)->last_send_time), &diff);
-                  host_timediff = (ARP_UINT64)1000000*diff.tv_sec +
+                  host_timediff = (uint64_t)1000000*diff.tv_sec +
                                   diff.tv_usec;
                   while (host_timediff >= (*cursor)->timeout && live_count) {
                      if ((*cursor)->live) {
@@ -572,7 +576,7 @@ main(int argc, char *argv[]) {
                         advance_cursor();
                      }
                      timeval_diff(&now, &((*cursor)->last_send_time), &diff);
-                     host_timediff = (ARP_UINT64)1000000*diff.tv_sec +
+                     host_timediff = (uint64_t)1000000*diff.tv_sec +
                                      diff.tv_usec;
                   }
                   first_timeout=0;
@@ -708,7 +712,11 @@ display_packet(host_entry *he, arp_ether_ipv4 *arpei,
       if (vendor)
          msg = make_message("%s\t%s", cp, vendor);
       else
-         msg = make_message("%s\t%s", cp, "(Unknown)");
+         /* Of the first octet of the address, check the second-least-significant bit */
+         if (arpei->ar_sha[0] & (1<<1))
+             msg = make_message("%s\t%s", cp, "(Unknown: locally administered)");
+         else
+             msg = make_message("%s\t%s", cp, "(Unknown)");
       free(cp);
 /*
  *	Check that any data after the ARP packet is zero.
@@ -975,7 +983,7 @@ usage(int status, int detailed) {
       fprintf(stdout, "<s> A character string, e.g. --file=hostlist.txt.\n");
       fprintf(stdout, "\n");
       fprintf(stdout, "<i> An integer, which can be specified as a decimal number or as a hexadecimal\n");
-      fprintf(stdout, "    number if preceeded with 0x, e.g. --arppro=2048 or --arpro=0x0800.\n");
+      fprintf(stdout, "    number if preceded with 0x, e.g. --arppro=2048 or --arpro=0x0800.\n");
       fprintf(stdout, "\n");
       fprintf(stdout, "<f> A floating point decimal number, e.g. --backoff=1.5.\n");
       fprintf(stdout, "\n");
@@ -1055,7 +1063,7 @@ usage(int status, int detailed) {
       fprintf(stdout, "\n--randomseed=<i>\tUse <i> to seed the pseudo random number generator.\n");
       fprintf(stdout, "\t\t\tThis option seeds the PRNG with the specified number,\n");
       fprintf(stdout, "\t\t\twhich can be useful if you want to ensure that the\n");
-      fprintf(stdout, "\t\t\trandom host list is reproducable. By default, the PRNG\n");
+      fprintf(stdout, "\t\t\trandom host list is reproducible. By default, the PRNG\n");
       fprintf(stdout, "\t\t\tis seeded with an unpredictable value. This option is\n");
       fprintf(stdout, "\t\t\tonly effective in conjunction with the --random (-R)\n");
       fprintf(stdout, "\t\t\toption.\n");
@@ -1078,7 +1086,7 @@ usage(int status, int detailed) {
       fprintf(stdout, "\t\t\tNo protocol decoding is performed and the OUI mapping\n");
       fprintf(stdout, "\t\t\tfiles are not used.\n");
       fprintf(stdout, "\n--plain or -x\t\tDisplay plain output showing only responding hosts.\n");
-      fprintf(stdout, "\t\t\tThis option supresses the printing of the header and\n");
+      fprintf(stdout, "\t\t\tThis option suppresses the printing of the header and\n");
       fprintf(stdout, "\t\t\tfooter text, and only displays one line for each\n");
       fprintf(stdout, "\t\t\tresponding host. Useful if the output will be\n");
       fprintf(stdout, "\t\t\tparsed by a script.\n");
@@ -1623,15 +1631,6 @@ recvfrom_wto(int sock_fd, int tmo, pcap_t *pcap_handle) {
    if (n < 0) {
       err_sys("select");
    } else if (n == 0 && sock_fd >= 0) {
-/*
- * For the BPF pcap implementation, we call pcap_dispatch() even if select
- * times out. This is because on many BPF implementations, select() doesn't
- * indicate if there is input waiting on a BPF device.
- */
-#ifdef ARP_PCAP_BPF
-      if ((pcap_dispatch(pcap_handle, -1, callback, NULL)) == -1)
-         err_sys("pcap_dispatch: %s\n", pcap_geterr(pcap_handle));
-#endif
       return;	/* Timeout */
    }
 /*
@@ -1985,7 +1984,7 @@ process_options(int argc, char *argv[]) {
 void
 arp_scan_version (void) {
    fprintf(stdout, "%s\n\n", PACKAGE_STRING);
-   fprintf(stdout, "Copyright (C) 2005-2016 Roy Hills, NTA Monitor Ltd.\n");
+   fprintf(stdout, "Copyright (C) 2005-2019 Roy Hills, NTA Monitor Ltd.\n");
    fprintf(stdout, "arp-scan comes with NO WARRANTY to the extent permitted by law.\n");
    fprintf(stdout, "You may redistribute copies of arp-scan under the terms of the GNU\n");
    fprintf(stdout, "General Public License.\n");
@@ -2424,7 +2423,7 @@ get_mac_vendor_filename(const char *specified_filename,
  *      Zero on success, or -1 on failure.
  */
 int
-get_source_ip(const char *interface_name, uint32_t *ip_addr) {
+get_source_ip(const char *interface_name, struct in_addr *ip_addr) {
    char errbuf[PCAP_ERRBUF_SIZE];
    pcap_if_t *alldevsp;
    pcap_if_t *device;
@@ -2453,10 +2452,12 @@ get_source_ip(const char *interface_name, uint32_t *ip_addr) {
       }
    }
    if (sin == NULL) {
+      memset(&(ip_addr->s_addr), '\0', sizeof(ip_addr->s_addr));
+      pcap_freealldevs(alldevsp);
       return -1;
    }
 
-   memcpy(ip_addr, &(sin->sin_addr.s_addr), sizeof(*ip_addr));
+   memcpy(ip_addr, &(sin->sin_addr), sizeof(*ip_addr));
 
    pcap_freealldevs(alldevsp);
 
