@@ -1,5 +1,5 @@
 /*
- * The ARP Scanner (arp-scan) is Copyright (C) 2005-2016 Roy Hills,
+ * The ARP Scanner (arp-scan) is Copyright (C) 2005-2019 Roy Hills,
  * NTA Monitor Ltd.
  *
  * This file is part of arp-scan.
@@ -17,14 +17,14 @@
  * You should have received a copy of the GNU General Public License
  * along with arp-scan.  If not, see <http://www.gnu.org/licenses/>.
  *
- * link-packet-socket.c -- Packet socket link layer send functions for arp-scan
+ * link-packet-socket.c -- Packet socket link layer functions for arp-scan
  *
  * Author:	Roy Hills
  * Date:	1 July 2006
  *
  * Description:
  *
- * This contains the link layer sending functions using the packet socket
+ * This contains the link layer functions using the packet socket
  * implementation.  Packet socket is typically used on Linux with kernel
  * version 2.2 and above.  See packet(7) on a Linux system for details.
  *
@@ -118,12 +118,16 @@ get_hardware_address(const char *if_name, unsigned char hw_address[]) {
    link_t *handle;
 
    handle = link_open(if_name);
+   if(!handle) {
+       err_sys("link_open");
+       return;
+   }
 
 /* Obtain hardware address for specified interface */
    if ((ioctl(handle->fd, SIOCGIFHWADDR, &(handle->ifr))) != 0)
       err_sys("ioctl");
 
-   link_close(handle);
-
    memcpy(hw_address, handle->ifr.ifr_ifru.ifru_hwaddr.sa_data, ETH_ALEN);
+
+   link_close(handle);
 }
